local frame = CreateFrame("FRAME"); -- Need a frame to respond to events
frame:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded
frame:RegisterEvent("PLAYER_LOGOUT"); -- Fired when about to log out
frame:RegisterEvent("PLAYER_REGEN_DISABLED");
frame:RegisterEvent("PLAYER_REGEN_ENABLED");
frame:RegisterEvent("TRADE_SKILL_UPDATE");

local dewdrop = LibStub('Dewdrop-2.0', true)    --Dropdownmenu
local _, L = ...                                --Localization
local _, P = ...                                --Professions (from professions.lua)
local me={}

me.combat = false                               --Are you in fight?	
me.default = {}
me.config = {
        showprimary = true,
        showsecondary = true,
        showother = true,
        showcds = true,
        moveable = false
}

me.sharedcds = {}                               --shared cds
if P.SharedCDs then
        for k, v in pairs(P.SharedCDs) do
         if GetSpellInfo(k) then --check if spell exists
          me.sharedcds[GetSpellInfo(k)] = v
         end
        end
end



--------------------------------------------------------------------------------
--GET SKILLS, WEAPON SKILLS, RANKS, ICONS, ...
--------------------------------------------------------------------------------
--Localize Profession / Include Icon / Set Secure or Disabled
function me:LocalizeProf(name, id)
        local spell = {}
        spell.name,_,spell.icon,_,_,_,_,_,_ = GetSpellInfo(id)
        if (strfind(name,",Secure")~=nil) then
         spell.secure = true
        elseif (strfind(name,",Disabled")~=nil) then
         spell.disabled = true
        elseif (strfind(name,",CD")~=nil) then --new
         spell.cd = true
        end
        spell.id = id
        return spell
end
--Localize List of all Professions in idarray and Creat Lines
function me:LocalizeList(profarray)--Professions ID-Array from professions.lua
        local k, v, kk, vv
        local result = {}
        local spell = {}
        for k, v in pairs(profarray) do
         if (type(v)=="number") then
          result = me:LocalizeProf(k,v)
          spell[result.name] = result
         else
          result = me:LocalizeProf(k,v["*"])
          spell[result.name] = result
          local help = result.name
          for kk, vv in pairs(profarray[k]) do
           if (kk~="*") then
            result = me:LocalizeProf(kk,vv)
            if (result.cd==true) then --is Spell CD?
             if (spell[help]["cd"]==nil) then
              spell[help]["cd"] = {}
             end
             spell[help]["cd"][result.name] = result.id
            else --no Spell CD
             if (spell[help]["more"]==nil) then
              spell[help]["more"] = {}
             end
             spell[help]["more"][result.name] = result
            end --end
           end
          end
         end
        end
        return spell
end
--Set New Table
P.Primary = me:LocalizeList(P.Primary)
P.Secondary = me:LocalizeList(P.Secondary)
--Filter professions and add Ranks
function me:FilterProfs(profarray) --Array from me:LocalizeList
        local spell = {}
	for i = 1, GetNumSkillLines() do
		local skillName, isHeader, _, skillRank, _, _, skillMaxRank = GetSkillLineInfo(i)
		if (isHeader == nil) then
		        if (profarray[skillName]~=nil) then
		         spell[skillName] = profarray[skillName]
		         spell[skillName]["name"] = nil
                         spell[skillName]["rank"] = skillRank
		         spell[skillName]["maxrank"] = skillMaxRank
		        end
		else
		      ExpandSkillHeader(i)
		end
	end
        return spell	
end
--Get Weapon Skills
function me:GetWeaponSkills()
        local skills = {}
        local active = false
        for i = 1, GetNumSkillLines() do
	 local skillName, isHeader, _, skillRank, _, _, skillMaxRank = GetSkillLineInfo(i)
	 if (isHeader==1) then
	  if (skillName==L["weapon"]) then
           active=true
           ExpandSkillHeader(i)
          else
           active=false
          end
	 else
          if (active == true) then
	   skills[skillName] = {}
           skills[skillName]["rank"] = skillRank
	   skills[skillName]["maxrank"] = skillMaxRank
	  end
	 end 
	end
        return skills
end



--------------------------------------------------------------------------------
--Table Alphabetical Sort Function
--------------------------------------------------------------------------------
function me:pairsByKeys (t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end



--------------------------------------------------------------------------------
--Format Cooldown Duration
--------------------------------------------------------------------------------
function me:formatduration(duration)
        local d,h,m
        d = floor(duration/86400)
        h = floor(duration/3600)-(d*24)
        m = floor(duration/60)-((d*1440)+(h*60))
        if d==0 then
         if h==0 then
          if m==0 then
           return "<1"..L["m"]
          else
           return m..L["m"]
          end
         else
          return h..L["h"].." "..m..L["m"]
         end
        else
         return d..L["d"].." "..h..L["h"]
        end 
end



--------------------------------------------------------------------------------
-- Load/Save Saved Variables
--------------------------------------------------------------------------------
function frame:OnEvent(event, arg1)
 	if event == "ADDON_LOADED" and arg1 == "Broker_ProfessionsMenu" then
		if (ProfessionsMenu == nil) then
			ProfessionsMenu = {}
		end
		if (ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")] == nil) then
		        ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")] = {}
		end
		if (ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["config"] == nil) then
			ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["config"] = {
				showprimary = true,
				showsecondary = true,
				showother = true,
				showcds = true
			}
		end
		if (ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["default"] == nil) then
			ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["default"] = {
                                left = "---",
				shiftleft = "---",
				altleft = "---",
				ctrlleft = "---",
				icon = "Interface\\Icons\\Trade_BlackSmithing"
			}
		end
		if (ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["cds"] == nil) then
		      ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["cds"] = {}
		end
		
                me.default = {
                        left = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["default"]["left"],
                        shiftleft = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["default"]["shiftleft"],
                        altleft = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["default"]["altleft"],
                        ctrlleft = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["default"]["ctrlleft"],
                        icon = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["default"]["icon"]
                }
		me.config = {
		        showprimary = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["config"]["showprimary"],
		        showsecondary = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["config"]["showsecondary"],
		        showother = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["config"]["showother"],
		        showcds = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["config"]["showcds"]
		}
		me.cds = ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")]["cds"]

		me:InitLDB()		--Init LibDataBroker

		me:Professions_UpdateInfo(me.default.left,me.default.icon)  -- Update Broker

	elseif event == "PLAYER_LOGOUT" then
   		ProfessionsMenu[UnitName("player").." - "..GetCVar("realmName")] = { -- We've met; commit it to memory.
			default = {
                                left           = me.default.left,
                                shiftleft      = me.default.shiftleft,
                                altleft        = me.default.altleft,
                                ctrlleft       = me.default.ctrlleft,
                                icon           = me.default.icon
                        },
                        config = {
			        showprimary    = me.config.showprimary,
			        showsecondary  = me.config.showsecondary,
			        showother      = me.config.showother,
			        showcds        = me.config.showcds
			},
			cds = me.cds
		}
	elseif event == "PLAYER_REGEN_DISABLED" then
	       me.combat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
	       me.combat = false
	elseif event == "TRADE_SKILL_UPDATE" then
	       if not IsTradeSkillLinked() and GetNumTradeSkills()~=nil then
	        for i = 1, GetNumTradeSkills() do
	         if skilltype~="header" then
	          local name = GetTradeSkillInfo(i)
                  local cd=GetTradeSkillCooldown(i)
                  if cd then
                   if me.sharedcds[name] then name=me.sharedcds[name] end
                   me.cds[name]=time()+floor(cd+0.5)
                  end
	         end
	        end 
	       end
	end
end
frame:SetScript("OnEvent", frame.OnEvent);



--------------------------------------------------------------------------------
-- Dropdownmenu
--------------------------------------------------------------------------------
function me:UpdateMenu(level, value)
	local info = {}
    	if not level then return end
    	if level == 1 then
    	
		me:AddMenu("title","|cff00ff00"..L["primary"].."|r")
		for k,v in me:pairsByKeys(me:FilterProfs(P.Primary)) do
			if (v.secure == true) then
			 me:AddMenu("securespell",k,nil,v.icon,k,nil,v.disabled,true)
			else
			 me:AddMenu("spell",k,nil,v.icon,k,nil,v.disabled,true)
			end
			if (v["more"]~=nil) then
			 for kk,vv in me:pairsByKeys(v["more"]) do
			  if (vv.secure == true) then
			   me:AddMenu("securespell","->"..kk,nil,vv.icon,kk,nil,vv.disabled,true)
			  else
			   me:AddMenu("spell","->"..kk,nil,vv.icon,kk,nil,vv.disabled,true)
			  end
			 end
			end
		end

		dewdrop:AddLine()
		me:AddMenu("title","|cff00ff00"..L["secondary"].."|r")
		for k,v in me:pairsByKeys(me:FilterProfs(P.Secondary)) do
			if (v.secure == true) then
			 me:AddMenu("securespell",k,nil,v.icon,k,nil,v.disabled,true)
			else
			 me:AddMenu("spell",k,nil,v.icon,k,nil,v.disabled,true)
			end
			if (v["more"]~=nil) then
			 for kk,vv in me:pairsByKeys(v["more"]) do
			  if (vv.secure == true) then
			   me:AddMenu("securespell","->"..kk,nil,vv.icon,kk,nil,vv.disabled,true)
			  else
			   me:AddMenu("spell","->"..kk,nil,vv.icon,kk,nil,vv.disabled,true)
			  end
			 end
			end
		end

		dewdrop:AddLine()

		me:AddMenu("arrow",L["settings"],"config")

	elseif level == 2 then --Settingsmenu
		if value == "config" then
			me:AddMenu("arrow",L["defaultprofession"],"default")
			
			if (me.config.showprimary == true) then info.checked = true else info.checked = nil end
			info.func = function()
				if (me.config.showprimary == true) then
					me.config.showprimary = false
				else
					me.config.showprimary = true
				end
			end
			me:AddMenu("text",L["showprimary"],nil,nil,nil,info.checked,nil,nil,info.func)

			if (me.config.showsecondary == true) then info.checked = true else info.checked = nil end
			info.func = function()
				if (me.config.showsecondary == true) then
					me.config.showsecondary = false
				else
					me.config.showsecondary = true
				end			
			end
			me:AddMenu("text",L["showsecondary"],nil,nil,nil,info.checked,nil,nil,info.func)
                        
                        --New function 6th July 2010
			if (me.config.showcds == true) then info.checked = true else info.checked = nil end
			info.func = function()
			        if (me.config.showcds == true) then
					me.config.showcds = false
				else
					me.config.showcds = true
				end
			end
			me:AddMenu("text",L["showcds"],nil,nil,nil,info.checked,nil,nil,info.func)
			--------------------------
                        
			if (me.config.showother == true) then info.checked = true else info.checked = nil end
			info.func = function()
				if (me.config.showother == true) then
					me.config.showother = false
				else
					me.config.showother = true
				end
			end
			me:AddMenu("text",L["showweapon"],nil,nil,nil,info.checked,nil,nil,info.func)
			
			dewdrop:AddLine()
			
			info.func = function()
			        me.cds = {}
			        me.secureframe:Hide()
			        GameTooltip:Hide()
				dewdrop:Close(1)
			end
			me:AddMenu("text",L["resetcds"],nil,nil,nil,nil,nil,nil,info.func)
			
			info.func = function()
			        me.config.moveable = true
			        me.secureframe:Hide()
			        GameTooltip:Hide()
				dewdrop:Close(1)
			end
			me:AddMenu("text",L["unlock"],nil,nil,nil,nil,nil,nil,info.func)
		end
        elseif level == 3 then
                me:AddMenu("arrow",L["leftclick"],"default1")
                me:AddMenu("arrow",L["shiftleftclick"],"default2")
                me:AddMenu("arrow",L["altleftclick"],"default3")
                me:AddMenu("arrow",L["ctrlleftclick"],"default4")
	elseif level == 4 then --Default Profession Menu
		if value == "default1" then --LEFTCLICK
		        if (me.default.left == "---") then info.checked = true else info.checked = nil end
					info.func = function()
					        me.default.left = "---"
						me.default.icon = "Interface\\Icons\\Trade_BlackSmithing"
						me:Professions_UpdateInfo(me.default.left,me.default.icon)
					end
			me:AddMenu("text","---",nil,nil,nil,info.checked,nil,nil,info.func)		
			dewdrop:AddLine()		
			for k,v in me:pairsByKeys(me:FilterProfs(P.Primary)) do
					if (me.default.left == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me:Professions_UpdateInfo(k,v.icon)
						me.default.left = k
						me.default.icon = v.icon
					end
					me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.left == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me:Professions_UpdateInfo(kk,vv.icon)
								me.default.left = kk
								me.default.icon = vv.icon
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
			dewdrop:AddLine()
			for k,v in me:pairsByKeys(me:FilterProfs(P.Secondary)) do
					if (me.default.left == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me:Professions_UpdateInfo(k,v.icon)
						me.default.left = k
						me.default.icon = v.icon
					end
				        me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.left == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me:Professions_UpdateInfo(kk,vv.icon)
								me.default.left = kk
								me.default.icon = vv.icon
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
		end
		if value == "default2" then --SHIFT + LEFT
		        if (me.default.shiftleft == "---") then info.checked = true else info.checked = nil end
					info.func = function()
					        me.default.shiftleft = "---"
					end
			me:AddMenu("text","---",nil,nil,nil,info.checked,nil,nil,info.func)		
			dewdrop:AddLine()		
			for k,v in me:pairsByKeys(me:FilterProfs(P.Primary)) do
					if (me.default.shiftleft == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me.default.shiftleft = k
					end
					me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.shiftleft == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me.default.shiftleft = kk
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
			dewdrop:AddLine()
			for k,v in me:pairsByKeys(me:FilterProfs(P.Secondary)) do
					if (me.default.shiftleft == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me.default.shiftleft = k
					end
				        me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.shiftleft == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me.default.shiftleft = kk
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
		end
		if value == "default3" then --ALT + LEFT
		        if (me.default.altleft == "---") then info.checked = true else info.checked = nil end
					info.func = function()
					        me.default.altleft = "---"
					end
			me:AddMenu("text","---",nil,nil,nil,info.checked,nil,nil,info.func)		
			dewdrop:AddLine()		
			for k,v in me:pairsByKeys(me:FilterProfs(P.Primary)) do
					if (me.default.altleft == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me.default.altleft = k
					end
					me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.altleft == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me.default.altleft = kk
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
			dewdrop:AddLine()
			for k,v in me:pairsByKeys(me:FilterProfs(P.Secondary)) do
					if (me.default.altleft == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me.default.altleft = k
					end
				        me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.altleft == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me.default.altleft = kk
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
		end
		if value == "default4" then --CTRL + LEFT
		        if (me.default.ctrlleft == "---") then info.checked = true else info.checked = nil end
					info.func = function()
					        me.default.ctrlleft = "---"
					end
			me:AddMenu("text","---",nil,nil,nil,info.checked,nil,nil,info.func)		
			dewdrop:AddLine()		
			for k,v in me:pairsByKeys(me:FilterProfs(P.Primary)) do
					if (me.default.ctrlleft == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me.default.ctrlleft = k
					end
					me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.ctrlleft == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me.default.ctrlleft = kk
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
			dewdrop:AddLine()
			for k,v in me:pairsByKeys(me:FilterProfs(P.Secondary)) do
					if (me.default.ctrlleft == k) then info.checked = true else info.checked = nil end
					info.func = function()
						me.default.ctrlleft = k
					end
				        me:AddMenu("text",k,nil,nil,nil,info.checked,v.disabled,nil,info.func)
					if (v["more"]~=nil) then
						for kk,vv in me:pairsByKeys(v["more"]) do
							if (me.default.ctrlleft == kk) then info.checked = true else info.checked = nil end
							info.func = function()
								me.default.ctrlleft = kk
							end
							if (vv.disabled == nil) then me:AddMenu("text","->"..kk.." ("..k..")",nil,nil,nil,info.checked,nil,nil,info.func) end
						end
					end
			end
		end
	end
end



--------------------------------------------------------------------------------
-- Tooltip
--------------------------------------------------------------------------------
-- All credit for this func goes to Tekkub and his picoGuild!
function me:GetTipAnchor(frame) --although used with dropdownmenu
		local x, y = frame:GetCenter()
		if not x or not y then return 'TOPLEFT', 'BOTTOMLEFT' end
		local hhalf = (x > UIParent:GetWidth() * 2 / 3) and 'RIGHT' or (x < UIParent:GetWidth() / 3) and 'LEFT' or ''
		local vhalf = (y > UIParent:GetHeight() / 2) and 'TOP' or 'BOTTOM'
		return vhalf..hhalf, frame, (vhalf == 'TOP' and 'BOTTOM' or 'TOP')..hhalf
end
-- Create Tooltip
function me:tooltip(self)
                GameTooltip:SetOwner(self, "ANCHOR_NONE")
                GameTooltip:SetPoint(me:GetTipAnchor(self))
	        GameTooltip:ClearLines()
                GameTooltip:AddLine(L["professions"].." (Broker_ProfessionsMenu)")
                                
                if (me.config.showprimary == true) then
                 GameTooltip:AddLine(" ")
                 GameTooltip:AddLine(L["primary"])
                 for k,v in me:pairsByKeys(me:FilterProfs(P.Primary)) do
                   GameTooltip:AddDoubleLine(k,v["rank"].."/"..v["maxrank"],1,1,1,0,1,0)
		 end
                end
                
                if (me.config.showsecondary == true) then
                 GameTooltip:AddLine(" ")
                 GameTooltip:AddLine(L["secondary"])
                 for k,v in me:pairsByKeys(me:FilterProfs(P.Secondary)) do
                  GameTooltip:AddDoubleLine(k,v["rank"].."/"..v["maxrank"],1,1,1,0,1,0)
                  if (v["cd"]~=nil and me.config.showcds) then
		   for kk,vv in me:pairsByKeys(v["cd"]) do
                    GameTooltip:AddDoubleLine("->"..kk,me:GetCD(vv),1,1,1,0,1,0)
		   end
		  end
                 end
                end
                
                if (me.config.showcds == true) then
                  local jj=nil
                  GameTooltip:AddLine(" ")
                  GameTooltip:AddLine(L["cds"])
                  for k,v in me:pairsByKeys(me.cds) do
                   local duration = difftime(v,time())
                   if duration>0 then
                    jj=true
                    GameTooltip:AddDoubleLine(k,me:formatduration(duration),1,1,1,1,0,0)
                   end
                  end
                  if not jj then
                   GameTooltip:AddLine(L["nocds"],0,1,0)
                  end
                end 
                
                if (me.config.showother == true) then
                 GameTooltip:AddLine(" ")
                 GameTooltip:AddLine(L["weapon"])
                 for k,v in me:pairsByKeys(me:GetWeaponSkills()) do
                  GameTooltip:AddDoubleLine(k,v["rank"].."/"..v["maxrank"],1,1,1,0,1,0)
                 end
                end
                
                GameTooltip:AddLine(" ")
	        GameTooltip:AddLine(L["leftclick"]..": |cffffffff"..me.default.left.."|r")
	        GameTooltip:AddLine(L["shiftleftclick"]..": |cffffffff"..me.default.shiftleft.."|r")
	        GameTooltip:AddLine(L["altleftclick"]..": |cffffffff"..me.default.altleft.."|r")
	        GameTooltip:AddLine(L["ctrlleftclick"]..": |cffffffff"..me.default.ctrlleft.."|r")
		GameTooltip:AddLine(L["rightclick"])
end
-- Create Tooltip for unlocked frame
function me:tooltipunlock(self)
	GameTooltip:SetOwner(self, "ANCHOR_NONE")
        GameTooltip:SetPoint(me:GetTipAnchor(self))
	GameTooltip:ClearLines()
        GameTooltip:AddLine(L["professions"].." (Broker_ProfessionsMenu)")
	GameTooltip:AddLine(" ")
        GameTooltip:AddLine(L["leftclick"]..": |cffffffff"..L["lock"].."|r")
end



--------------------------------------------------------------------------------
-- Init LibDataBroker
--------------------------------------------------------------------------------
function me:InitLDB()
--Create Secure Frame Overlay
me.secureframe = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate");
me.secureframe:SetScript("OnLeave",function() me.secureframe:Hide(); GameTooltip:Hide() end)
me.secureframe:SetScript("OnEnter",function(self) me:tooltip(self); GameTooltip:Show() end)
me.secureframe:SetScript("OnMouseDown",function(self,button)
                                me.secureframe:SetAttribute("type", nil)
                                me.secureframe:SetAttribute("spell", nil)
                                if button == "LeftButton" then
                                        if me.combat then
                                                me:Error(L["infight"])
                                                return
                                        end
                                        if IsControlKeyDown() then
                                                if (me.default.ctrlleft == "---") then
                                                        me:Error(L["noprofession"])
                                                        return
                                                end
                                                me.secureframe:SetAttribute("spell", me.default.ctrlleft)
                                        elseif IsAltKeyDown() then
                                                if (me.default.altleft == "---") then
                                                        me:Error(L["noprofession"])
                                                        return
                                                end
                                                me.secureframe:SetAttribute("spell", me.default.altleft)
                                        elseif IsShiftKeyDown() then
                                                if (me.default.shiftleft == "---") then
                                                        me:Error(L["noprofession"])
                                                        return
                                                end
                                                me.secureframe:SetAttribute("spell", me.default.shiftleft)
                                        else
                                                if (me.default.left == "---") then
                                                        me:Error(L["noprofession"])
                                                        return
                                                end
                                                me.secureframe:SetAttribute("spell", me.default.left)
                                        end
                                        me.secureframe:SetAttribute("type", "spell")
                                end
                                if button == "RightButton" then
                                        GameTooltip:Hide()
                                        dewdrop:Open(self, 'children', function(level, value) me:UpdateMenu(level, value) end)
                                end
                        end)
        
ldb = LibStub:GetLibrary("LibDataBroker-1.1");
me.dataobj = ldb:NewDataObject("Broker_ProfessionsMenu", {
	type = "data source",
	label = nil,
	text = nil,
	icon = "Interface\\Icons\\Trade_BlackSmithing",
	OnClick = function(self,btn)
	        if btn == "LeftButton" then
	               me.config.moveable = false
	               GameTooltip:Hide()
	        end
	end,
        OnEnter = function(self)    
                if me.config.moveable then
                 me:tooltipunlock(self)
                 GameTooltip:Show()
                 return 
                end 
                me.secureframe:SetFrameStrata(self:GetFrameStrata())
                me.secureframe:SetFrameLevel(self:GetFrameLevel()+1)                       
                me.secureframe:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT",0,0)
                me.secureframe:SetWidth(self:GetWidth())
                me.secureframe:SetHeight(self:GetHeight()) 
                me.secureframe:Show()            
	end,
	OnLeave = function(self)
		GameTooltip:Hide()
	end
})
end



--------------------------------------------------------------------------------
-- Refresh broker icon/text
--------------------------------------------------------------------------------
function me:Professions_UpdateInfo(txt,icon)
	if (txt ~= nil) then
		me.dataobj.text = txt
	end
	if (icon ~= nil) then
		me.dataobj.icon = icon
	end
end



--------------------------------------------------------------------------------
-- Error Messages
--------------------------------------------------------------------------------
function me:Error(txt)
        UIErrorsFrame:AddMessage(txt,1.0,0,0,100,3);
end



--------------------------------------------------------------------------------
-- Dewdrop funtions
--------------------------------------------------------------------------------
function me:AddMenu(type,text,value,icon,spell,checked,disabled,closeWhenClicked,onclick)
if (type == "securespell") then
	local secure = {type = "spell",
			spell = spell}
	dewdrop:AddLine(
		'text', text,
		'icon', icon, 
		'secure', secure,
		'checked', checked,
		'disabled', disabled,
		'closeWhenClicked', closeWhenClicked,
		'tooltipFunc', function() return end
	)
elseif (type == "spell") then
	dewdrop:AddLine(
		'text', text,
		'icon', icon,
		'checked', checked,
		'disabled', disabled,
		'func', function() CastSpellByName(spell) end,
		'closeWhenClicked', closeWhenClicked,
		'tooltipFunc', function() return end
	)
elseif (type == "title") then
	dewdrop:AddLine(
      		'text', text,
		'icon', icon,
      		'isTitle', true,
      		'tooltipFunc', function() return end
    	)
elseif (type == "text") then
	dewdrop:AddLine(
                'text', text,
		'icon', icon,
		'checked', checked,
		'disabled', disabled,
		'func', onclick,
		'closeWhenClicked', closeWhenClicke,
		'tooltipFunc', function() return end
    	)
elseif (type == "arrow") then
	dewdrop:AddLine(
		'text', text,
		'hasArrow', true,
		'disabled', disabled,
		'value', value,
		'tooltipFunc', function() return end
	)
end
end